#pragma config OSC=HSPLL
#pragma config FCMEN=OFF
#pragma config IESO=OFF
#pragma config WDTPS=1024
// #pragma config WDTPS=8192
#pragma config PWRT=OFF
#pragma config LPT1OSC=OFF
#pragma config BORV=3		
#pragma config STVREN=ON
#pragma config DEBUG=OFF
#pragma config WRTB=ON
#pragma config PBADEN=OFF
#pragma config LVP=OFF

#include <p18f4620.h> 
//#include <reset.h>
#include <delays.h>


#include "leds.h"
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "TCPIP Stack\MAC.h"
#include "TCPIP Stack\Helpers.h"
#include "TCPIP Stack\TCPIP.h"

#include "router.h"
#include "types.h"
#include "rules.h"
#include "syslog.h"



//
// PIC18 Interrupt Service Routines
// 
// NOTE: Several PICs, including the PIC18F4620 revision A3 have a RETFIE FAST/MOVFF bug
// The interruptlow keyword is used to work around the bug when using C18
	
#pragma interruptlow LowISR
void LowISR(void)
{
    TickUpdate();
}

#pragma interruptlow HighISR
void HighISR(void)
{	  
	//Check which interrupt flag caused the interrupt.
	if (PIR1bits.TMR1IF)
	  {
		  /* The TMR1 register pair (TMR1H:TMR1L) increments
       * from 0000h to FFFFh and rolls over to 0000h. The
       * Timer1 interrupt, if enabled, is generated on overflow,
       * which is latched in interrupt flag bit, TMR1IF
       * (PIR1<0>). This interrupt can be enabled or disabled
       * by setting or clearing the Timer1 Interrupt Enable bit,
       * TMR1IE (PIE1<0>).  
       * PIR1bits.TMR1IF = 0;
       */		    		
		}
}


#pragma code lowVector=0x18
void LowVector(void){_asm goto LowISR _endasm}

#pragma code highVector=0x8
void HighVector(void){_asm goto HighISR _endasm}


   
#pragma code 

void main(void)
  {
    WORD ip_total_length;
    unsigned int block;
    BYTE i;

    // CSX is named after the Carrier Select pin for SPI.
    // In this project, there are multiple devices sharing the SPI
    // bus.  The X references the device number.
    BYTE CSX;
    
    // The eth_type is the ether type field from the MAC layer.
    BYTE eth_type;

    // Store the register value for interrupts.
    REG reg;
    
    
    /* Beginning of main program */
    
    // initialize the leds
    LED_INIT;        
    
    // flash leds        
    LED_ON;   LED_PAUSE; 

    // This flash indicates that the Microcontroller is operational.  
    // No SPI communication is yet established, so something like power
    // or oscillator is at fault if the leds do not flash.
    LED_PAUSE; LED_OFF; LED_G_ON;   
      
    // Enable Global Interrupts 
    // TickGet() will hang if this is not enabled.
	  RCONbits.IPEN = 1;		// Enable interrupt priorities
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;
     
    // More LEDs to facilitate debugging. 
    LED_PAUSE; LED_OFF; LED_F_ON; 
    
    ClrWdt(); 
            
    // initilize time keeping system
    TickInit();
         
    LED_PAUSE; LED_OFF; LED_F_ON; LED_G_ON;
                   
    // read in the interface definitions, the route definitions and the access rules
    initilize_rule_data();                               
    
    LED_PAUSE; LED_OFF; LED_E_ON;
        
    // initilize the route cache
    route_cache_init();

    LED_PAUSE; LED_OFF; LED_G_ON; LED_E_ON;
        
    // I don't really use this yet
    srand(GenerateRandomDWORD());
    
    LED_PAUSE; LED_OFF;        
   
    // Initilize the Ethernet controllers
    // Communication is established via the SPI bus to
    // the ENC28J60 chips.  
    for (i=0u; i<NOI; i++)
      { 
        ClrWdt(); 
        LED_G_ON;
        MACInit(i, INTF_MAC_ADDR(i) );  
        
        // the LED display indicates that interface i was initialized.
        led_display(i);        
        setInterruptsEIE(i,
          	      EIE_INTIE  //Global INT Interrupt Enable bit
          	    | EIE_RXERIE //Receive Error Interrupt (RXERIF)
              //| EIE_TXERIE //Transmit Error Interrupt (TXERIF)
              //| EIE_TXIE   //Transmit Interrupt (TXIF)
                | EIE_LINKIE //Link Change Interrupt (LINKIF)
              //| EIE_DMAIE  //DMA Interrupt (DMAIF)
              //| EIE_PKTIE  //Receive Packet Pending Interrupt (PKTIF)    
            );
      }                                                        
 
    LED_PAUSE;
    
    ClrWdt(); 

    // This will (quickly) flash the Revision ID of the ENC28J60
    // Use this information to reference the Errata for
    // your chip's version.  I highly recommend this!
    for (i=0u; i<NOI; i++)
      {
        led_display(getRevID(i));  LED_PAUSE;  LED_OFF;  LED_PAUSE;
      }
    
    // initialize the ARP table.
    ARPInit();
      
    // CSX will iterate through all the interfaces.
    CSX = 0;
    
    // context is defined in TCPIP.h, declaired as extern in router.h and instantiated in router.c
    memset(context,0,sizeof(context));
    
    
#if defined (ENABLE_SYSLOG)

    syslog_init();
 
    // if syslog is enabled, this will send the revision id to the syslog server.
    sprintf(SYSLOG_PTR,"picsw: ENC28J60 RevID=(%d,%d,%d)",getRevID(0),getRevID(1),getRevID(2) );
    syslogit();

#endif
    
    
    while(1)
      {        
        LED_OFF;
        ClrWdt(); 
        
        CSX   = (++CSX)%NOI; // NOI is the number of interfaces.

#if defined (ENABLE_SYSLOG)        

        if (isInterrupt(CSX)) 
          {
            i = 0;
            reg =  getInterruptsEIF(CSX);
            
            /*
             *  EIE_RXERIE //Receive Error Interrupt (RXERIF)
             *  EIE_TXERIE //Transmit Error Interrupt (TXERIF)
             *  EIE_TXIE   //Transmit Interrupt (TXIF)
             *  EIE_LINKIE //Link Change Interrupt (LINKIF)
             *  EIE_DMAIE  //DMA Interrupt (DMAIF)
             *  EIE_PKTIE  //Receive Packet Pending Interrupt (PKTIF)    
             */
             
            //Link Change Interrupt (LINKIF)
            if (reg.EIRbits.LINKIF)      // !! Does not work !!
              {
                i |= EIR_LINKIF;
                sprintf(SYSLOG_PTR,"picsw: ENC28J60 Link event on CSX=%d",CSX);
                syslogit();
              }  
                      
            //Receive Error Interrupt (RXERIF)  
            if (reg.EIRbits.RXERIF)
              {
                i |= EIR_RXERIF;
                sprintf(SYSLOG_PTR,"picsw: ENC28J60 Lost RX packets on CSX=%d",CSX);
                syslogit();
              } 
              
            clearInterruptsEIR(CSX,i);
          } // if isInterrupt
#endif
                
        // if the source link is down then there is no need to see if any data is on this link
        if (! MACIsLinked(CSX)) continue;
        
        // if this interface (CSX) is waiting for an ARP to be returned on another interface, then don't do
        // anything to this interface until the arp is complete.  
        // NOTE: This is an area for future improvement!
        if (context[CSX].waiting_for_arp) 
          {
            LED_A_ON; //Waiting for ARP
            
            // At this point, the interface CSX is in a blocking state.  It has data cached and ready to
            // be sent, but we don't know the MAC address of the host to send it to. 
            // The arp was issued form the context[CSX].CSX_dst
			      if(ARPIsResolved(context[CSX].CSX_dst, &context[CSX].mac_dst.IPAddr, &context[CSX].mac_dst.MACAddr))
              {
                // The ARP is now resolved.  The when the mac_dst.IPAddr and mac_dst.MACAddr contain
                // the information for the remote host.
                 
                // Note that mac_dst.IPAddr is the IP of the host to which we hand this packet.  It is
                // not necessairly the same IP used in the IP header.
                
                // Route the packet of size "payloadlen" defiend in "header" from "mac_src" via "CSX" to "mac_dst" via "CSX_dst". 
                route_packet(context[CSX].CSX_dst, CSX, &context[CSX].mac_dst, &context[CSX].mac_src, &context[CSX].header, context[CSX].payloadlen);
                route_cache_add(CSX);                
                // Take this interface out of blocking and clear out the old data
                memset((void*) &context[CSX], 0, sizeof(CSX_CONTEXT)); 
              }
            else
              {                
                // If the ARP is not resolved.  Then we will issue another ARP every ARP_CYCLE time period until we
                // reach the ARP_LIMIT number of tries.
                if (TickGet() - context[CSX].arp_timer > ARP_CYCLE)
                  {
            
                    // It is time to issue another ARP or to free up this interface.
                    if (context[CSX].arp_count++ > ARP_LIMIT)
                      {
                        LED_G_ON; // dropped packet
                        
                        // When arp_count is greater than ARP_LIMIT we will discard this packet and free up this interface.
                        MACDiscardRx(CSX);
                        
                        // clear out the old data
                        memset((void*) &context[CSX], 0, sizeof(CSX_CONTEXT));                        
                      }
                    else
                      {
                        // Let's issue another arp request over the CSX_dst interface.
                        ARPResolve(context[CSX].CSX_dst, &context[CSX].mac_dst.IPAddr);
                        
                        // update the timer.
                        context[CSX].arp_timer = TickGet();
                      }                    
                  } // ARP_CYCLE
              } // else
              
            continue; // skip the rest of the while(1) loop.
          } // waiting for arp



        // See if any data is ready on the CSX device                                    
        if (MACGetHeader2(CSX, &(context[CSX].mac_dst.MACAddr), &(context[CSX].mac_src.MACAddr), &eth_type))
          {        
            // determine whether this is IP or ARP data.
            switch (eth_type) // determine the type of the packet.
              {
                case MAC_IP:                
                  // Was this packet sent to our MAC address? 
                  // MAC_CMP returns 0 if the two MAC addresses are the same.
                  // We will discard the IP packet if it is not directed to our MAC address.
                  // NOTE: This is now filtered in the Ethernet controller chip.
  			          // if (MAC_CMP( &context[CSX].mac_dst.MACAddr, intf[CSX].mac)) break;

                  // Since this is an IP packet, get the IP header information.
                  // NOTE: This function ignores all IP options    
          				if(!IPGetHeader(CSX, &context[CSX].header, &context[CSX].payloadlen)) break;          			
      
                  // If a route cache entry exists, then this command will route the packet
                  if (route_cache_exec(CSX)) break;

          				// Check for invalid src and dst address
          			  if (!validate_address(CSX, &context[CSX].header)) break;
          			  
                  LED_B_ON;          			  
                  
          			  // Populate the NODE_INFO structures with IP information from the
          			  // IP packet.  This may be modified later on in the program.
                  context[CSX].mac_dst.IPAddr = context[CSX].header.DestAddress;
                  context[CSX].mac_src.IPAddr = context[CSX].header.SourceAddress;
                                     			  
          	      // Is the packet for the router interface itself?
                  if (context[CSX].header.DestAddress.Val == intf[CSX].ip.Val)
                    { 
                      // What kinds of packets does the interface accept?
                      if (context[CSX].header.Protocol == IP_PROT_ICMP)
                        {
                          // We do respond to ICMP echo requests.                               
                          // The source will be the default for this interface.
                          // The response should be directed to the source of this packet.
                          ICMPProcess(CSX, &context[CSX].mac_src, context[CSX].payloadlen);
                        }
                      break;
                    }
                                  
                  // Now comes the fun stuff!  Where shall we send this packet.
                  // Previously, the mac_dst and mac_src were set to default values.
                  // The set_route function will assign new values, when necessary, to the
                  // mac_dst and mac_src.  These values will determine where to send the packet
                  // on the MAC layer.  The IP addresses inside the packet will not be modified.
                  // The set_route function will also set the CSX_dst value to the apropriate CSX value.
                  
                  // Remember that this is a first stab at routing code.  There is much, much room for improvement.
                  
                  if (set_route(CSX, &context[CSX].mac_dst, &context[CSX].mac_src))
                    {        
                      // check to see if we have a MAC address for the destination IP address.
          			      if(ARPIsResolved(context[CSX].CSX_dst,
          			                       &context[CSX].mac_dst.IPAddr, 
          			                       &context[CSX].mac_dst.MACAddr))
                        {           
                          // We do have an ARP entry.
                          // Now, send the packet on.
                          route_packet(context[CSX].CSX_dst, 
                                       CSX, 
                                       &context[CSX].mac_dst, 
                                       &context[CSX].mac_src, 
                                       &context[CSX].header, 
                                       context[CSX].payloadlen);
                          route_cache_add(CSX);
                        }
                      else
                        {
                          /*  RFC1812
                           *  The link layer MUST NOT report a Destination Unreachable error to IP
                           *  solely because there is no ARP cache entry for a destination; it
                           *  SHOULD queue up to a small number of datagrams breifly while
                           *  performing the ARP request/reply sequence, and reply that the
                           *  destination is unreachable to one of the queued datagrams only when
                           *  this proves fruitless.  
                           */

                          // We will have to wait for an arp entry to come available.
                          // Issue an ARP request over the CSX_dst interface. 
                          ARPResolve(context[CSX].CSX_dst, &context[CSX].mac_dst.IPAddr);

                          // Now, we will put this interface in blocking mode until we have and ARP entry
                          // or until we give up and discard the packet.
                          context[CSX].waiting_for_arp = 1;
                          context[CSX].arp_count=0;
                          context[CSX].arp_timer=TickGet(); 
                        }
                    }
                  break;

                case MAC_ARP:  
                    // This is an ARP packet.  
                    // It would be an ARP request or an ARP reply.
                    LED_C_ON;
                    ARPProcess(CSX);
                  break;                      
                
              } // switch 
              
            // Unless we are waiting for an ARP reply, we can safely clear clear out old data
            if (! context[CSX].waiting_for_arp)
              {        
                // reset the context
                memset((void*) &context[CSX], 0, sizeof(CSX_CONTEXT));
                
                // discard buffered data in the ENC28J60 chip
                MACDiscardRx(CSX); 
              }
        
         } // GetHeader2()

          
      } // while(1)

  } // main()

