#include "syslog.h"
#include "router.h"

#if defined (ENABLE_SYSLOG)

#define SYSLOG_CYCLE (1ul*TICK_SECOND)

static NODE_INFO syslog_node_info;
BYTE SYSLOG_READY = 0;
BYTE syslog[SYSLOG_BUFFER_SIZE];


  
// get the MAC address of the syslog server
BOOL syslog_init()
  {    
    BYTE eth_type;
    TICK syslog_timer = 0;    
    int attempts = 5;
    
    // copy IP address of syslog server
    syslog_node_info.IPAddr.Val = syslog_server->ip.Val;
    SYSLOG_CLEAR;
    
    while (1)
      {    
        ClrWdt();        
        if (TickGet() - syslog_timer > SYSLOG_CYCLE)
          {
            ARPResolve(syslog_server->ino,&syslog_node_info.IPAddr);
            syslog_timer = TickGet();
            
            if (--attempts < 0) return FALSE;
          }
          
        if (MACGetHeader2(syslog_server->ino, 
                        &(context[syslog_server->ino].mac_dst.MACAddr), 
                        &(context[syslog_server->ino].mac_src.MACAddr), 
                        &eth_type))
          {        
            if (eth_type == MAC_ARP)
              {
                  ARPProcess(syslog_server->ino);
              }
            MACDiscardRx(syslog_server->ino);
          }
          
        if(ARPIsResolved(syslog_server->ino, &syslog_node_info.IPAddr, &syslog_node_info.MACAddr)) 
          {
            SYSLOG_READY = 1;
            return TRUE;        
          }
      }
    return FALSE;
  }
  
BOOL syslogit()
  {
    int len;
    
    if (! SYSLOG_READY) return FALSE;
    
    len = syslog_format();
    
	  while(!MACIsTxReady(syslog_server->ino));
	  MACSetWritePtr(syslog_server->ino,BASE_TX_ADDR);
	
    IPPutHeader(syslog_server->ino,  
                 &syslog_node_info,    // MAC address of packet
                 IP_PROT_UDP,  // MAC address of packet
                 len);    

    MACPutArray(syslog_server->ino, syslog, len);
  
    MACFlush(syslog_server->ino);
    return TRUE;
}    
    
WORD syslog_format()
  {
    WORD_VAL len;
    
    len.Val = strlen(SYSLOG_PTR);
    len.Val = ((WORD)8)+len.Val;

    syslog[0] = 0x01;   // from port 257
    syslog[1] = 0x01;
    
    syslog[2] = 0x02;   // to port 514
    syslog[3] = 0x02;  
        
    syslog[4] = len.v[1];   // length
    syslog[5] = len.v[0];  
                  
    syslog[6] = 0x00;   // leave checksum blank         
    syslog[7] = 0x00;         
        
    return len.Val;
  }
  
  
#endif       
    

