package FW::CIDR;

use strict;

use Socket;

sub cidr_reduce
  {
    # apply the bit mask to the ip address
    # return the network, mask, and number of bits.

    my $addr = shift;

    my ($ip,$bits) = ($addr =~ /(\d+\.\d+\.\d+\.\d+)\/(\d+)/);

    my $mask = cidr2mask( $bits );

    my @ip = split(/\./,$ip);
    my @mask = split(/\./,$mask);

    # apply CIDR mask to ip to eliminate extraneous precission
    for (my $i=0; $i<4; $i++)
      {
        $ip[$i] = unpack('C1',pack('C1',$ip[$i]) & pack('C1',$mask[$i]));
      }

    $ip = join(".",@ip);
    $mask = join(".",@mask);
    return ($ip,$mask,$bits);
  }

sub is_ip_in_network
  {
    my $ip = shift;
    my $net = shift;
    my $mask = shift;

    return ( bits2ip(ip2bits($ip) & ip2bits($mask)) eq $net);
  }

sub ip2bits
  {
    my $ip = shift;

    my @ip = split(/\./,$ip);

    my $bits = pack('C1 C1 C1 C1',@ip);
    return $bits;
  }

sub bits2ip
  {
    my $bits = shift;

    return join(".",unpack('C1 C1 C1 C1',$bits));
  }

sub cidr_intx
  {
    my $ip1 = shift;
    my $ip2 = shift;

    my ($a,$mask1,$bits1) = cidr_reduce($ip1);
    my ($b,$mask2,$bits2) = cidr_reduce($ip2);

    return 1 if (($a eq $b) && ($bits1 == $bits2));  # equal
    # return 0 if (($a eq $b) && ($bits1 == $bits2));

    print "($ip1,$bits1)($ip2,$bits2)\n";

    my $inet_ip1 = ip2bits($a);
    my $inet_ip2 = ip2bits($b);
   
    my $ab = bits2ip($inet_ip1 | $inet_ip2);
    
 print "A:  $a / $bits1\n";
 print "B:  $b / $bits2\n";
 print "AB: $ab\n\n";

    if (($a eq $ab) && ($b ne $ab))
      {
        # b is contained in a
        return 1;
      }

    if (($b eq $ab) && ($a ne $ab))
      {
        # a is contained in b
        return 1;
        # return -1;
      }

    # Hmm... they are the same.  Use the bits to break the tie.
    # the higher bits is more restrictive
    
    if ($bits1 < $bits2)
      {
        # then a is the larger set
        return +1;
      }

    if ($bits1 > $bits2)
      {
        # then b is the larger set
        return +1;
      }

    return 0;
  }


sub cidr2mask
  {
    my $value = shift;

    return "255.255.255.255" if ($value == 32);
    return "255.255.255.254" if ($value == 31);
    return "255.255.255.252" if ($value == 30);
    return "255.255.255.248" if ($value == 29);
    return "255.255.255.240" if ($value == 28);
    return "255.255.255.224" if ($value == 27);
    return "255.255.255.192" if ($value == 26);
    return "255.255.255.128" if ($value == 25);
    return "255.255.255.0" if ($value == 24);
    return "255.255.254.0" if ($value == 23);
    return "255.255.252.0" if ($value == 22);
    return "255.255.248.0" if ($value == 21);
    return "255.255.240.0" if ($value == 20);
    return "255.255.224.0" if ($value == 19);
    return "255.255.192.0" if ($value == 18);
    return "255.255.128.0" if ($value == 17);
    return "255.255.0.0" if ($value == 16);
    return "255.254.0.0" if ($value == 15);
    return "255.252.0.0" if ($value == 14);
    return "255.248.0.0" if ($value == 13);
    return "255.240.0.0" if ($value == 12);
    return "255.224.0.0" if ($value == 11);
    return "255.192.0.0" if ($value == 10);
    return "255.128.0.0" if ($value == 9);
    return "255.0.0.0" if ($value == 8);
    return "254.0.0.0" if ($value == 7);
    return "252.0.0.0" if ($value == 6);
    return "248.0.0.0" if ($value == 5);
    return "240.0.0.0" if ($value == 4);
    return "224.0.0.0" if ($value == 3);
    return "192.0.0.0" if ($value == 2);
    return "128.0.0.0" if ($value == 1);
    return "0.0.0.0" if ($value == 0);
  }


return 1;
