package FW::Route;

use strict;

use base qw(FW::Base);

use FW::CIDR;

sub new
  {
    my $pkg = shift;
    my $text = shift;
 
    my $obj = $pkg->SUPER::new($text);
    $obj->{'NET'} = '';
    $obj->{'BITS'} = '';
    $obj->{'MASK'} = '';
    $obj->{'DEFAULT'} = 0;

    $obj->{'INTERFACE'} = 0;
    $obj->{'INTERFACE_DEFINED'} = 0;
    $obj->{'REMOTE_IP'} = '';

    bless $obj, $pkg;

    $obj->expand($text) if ($text);

    return $obj;
  }

sub setNetwork
  {
    my $obj = shift;
    my $addr = shift;

    if ($addr eq '0.0.0.0/0')
      {
        $obj->{'DEFAULT'} = 1;
        $obj->{'BITS'} = '0';
        $obj->{'NET'}  = '0.0.0.0';
        $obj->{'MASK'} = '0.0.0.0';
        return;
      }

    my ($net,$mask,$bits) = FW::CIDR::cidr_reduce($addr);

    $obj->{'BITS'} = $bits;
    $obj->{'NET'}  = $net;
    $obj->{'MASK'} = $mask;
  }

sub getBits
  {
    my $obj = shift;

    return $obj->{'BITS'};
  }

sub setRemoteIP
  {
    my $obj = shift;
    $obj->{'REMOTE_IP'} = shift;
  }

sub getRemoteIP
  {
    my $obj = shift;
    return $obj->{'REMOTE_IP'};
  }

sub isInterfaceDefined
  {
    my $obj = shift;
    return $obj->{'INTERFACE_DEFINED'} ;
  }

sub getInterface
  {
    my $obj = shift;
    return $obj->{'INTERFACE'};
  }

sub setInterface
  {
    my $obj = shift;
    $obj->{'INTERFACE'} = shift;
    $obj->{'INTERFACE_DEFINED'} = 1;
  }

sub toString
  {
    my $obj = shift;

    my @ret = ();

    die "Interface is not defiend" unless ($obj->{'INTERFACE_DEFINED'});
    push(@ret,split(/\./,$obj->{'NET'}));
    push(@ret,split(/\./,$obj->{'MASK'}));

    push(@ret,$obj->{'INTERFACE'});
    push(@ret,split(/\./,$obj->{'REMOTE_IP'}) );

    return join(" ", $obj->toHex( @ret ));
  }

return 1;
