#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "../firmware/GenericTypeDefs.h"

// NOI = Number of Interfaces (how man CSX's)
#define NOI 3

// number of entries for each interface (each CSX)
#define ARP_TABLE_LENGTH 10

#define IP_ADDR		DWORD_VAL

typedef struct {
    BYTE v[6];
  } MAC_ADDR;

typedef struct __attribute__((__packed__)) _NODE_INFO
{
    IP_ADDR     IPAddr;
    MAC_ADDR    MACAddr;
} NODE_INFO;

// a single entry has storage for each CSX
struct ARP_TABLE_COLUMN {
  NODE_INFO entry[ARP_TABLE_LENGTH];
};

// this is the arp table
struct ARP_TABLE {
   BYTE pointer[NOI];
   struct ARP_TABLE_COLUMN csx[NOI];
};


struct ARP_TABLE arp_table;

BYTE ARPIsResolved(BYTE CSX, IP_ADDR *ip, MAC_ADDR *mac)
  {
    int i;
    for (i=0; i<ARP_TABLE_LENGTH; i++)
      {
        if (arp_table.csx[CSX].entry[i].IPAddr.Val == ip->Val)
          {
            if (mac)
              *mac = arp_table.csx[CSX].entry[i].MACAddr;
            return (1+i);
          }
      }
    return 0;
  }


void ARPInit(void)
  {
    int i,j;

    for (j=0; j<NOI; j++)
      {
        arp_table.pointer[j]=0;
        for (i=0; i<ARP_TABLE_LENGTH; i++)
          {
            memset((void*) &(arp_table.csx[j].entry[i].MACAddr),0xFF,sizeof(MAC_ADDR));
            arp_table.csx[j].entry[i].IPAddr.Val = 0ul;
          }
      }
  }

void arp_add_entry(BYTE CSX, IP_ADDR *ip, MAC_ADDR *mac)
  {
    int i = 0;

    i = ARPIsResolved(CSX, ip, NULL );

    if (i)
      i--;
    else
      {
        i = arp_table.pointer[CSX]++;
        arp_table.pointer[CSX] = arp_table.pointer[CSX] % ARP_TABLE_LENGTH;
      }

    arp_table.csx[CSX].entry[i].IPAddr = *ip;
    arp_table.csx[CSX].entry[i].MACAddr = *mac;
  }


void arp_print_table()
  {
    int i,j;

    for (i=0; i<ARP_TABLE_LENGTH; i++)
      {
        printf("(%d)\t",i);
        for (j=0; j<NOI; j++)
          printf("%x/%x\t", arp_table.csx[j].entry[i].IPAddr, arp_table.csx[j].entry[i].MACAddr.v[0]);
        printf("\n");
      }
    printf("\n");
  }



int main(int argc, char *argv[])
  {
    IP_ADDR ip;
    MAC_ADDR mac;

    int i;
    ARPInit();

    for (i=0; i<6; i++)
      {
        ip.Val = i;
        mac.v[0] = i+16;
        arp_add_entry(0, &ip, &mac);
      }
    arp_print_table();

    for (i=0; i<12; i++)
      {
        ip.Val = i;
        mac.v[0] = i+16;
        arp_add_entry(1, &ip, &mac);
      }
    arp_print_table();

    for (i=0; i<18; i++)
      {
        ip.Val = i;
        mac.v[0] = i+16;
        arp_add_entry(2, &ip, &mac);
      }
    arp_print_table();

    ip.Val = 3;
    mac.v[0] = 0x33;

    arp_add_entry(1, &ip, &mac);
    arp_add_entry(2, &ip, &mac);
    arp_add_entry(3, &ip, &mac);

    arp_print_table();


    // to set an entire colun to 0 
    memset((void*) &arp_table.csx[1],0,sizeof(struct ARP_TABLE_COLUMN));
    arp_print_table();
    return 0;
  }



